<?php

declare(strict_types=1);

namespace popp\r04\zestaw11;

class Conf
{
    private $xml;
    private $lastmatch;

    public function __construct(private string $file)
    {
        if (! file_exists($file)) {
            throw new FileException("Plik '$file' nie istnieje");
        }
        $this->xml = simplexml_load_file($file, null, LIBXML_NOERROR);
        if (! is_object($this->xml)) {
            throw new XmlException(libxml_get_last_error());
        }
        $matches = $this->xml->xpath("/conf");
        if (! count($matches)) {
            throw new ConfException("Nie można odnaleźć elementu źródłowego: conf");
        }
    }

    public function write(): void
    {
        if (! is_writeable($this->file)) {
            throw new \Exception("Plik '{$this->file}' nie jest dostępny do zapisu");
        }
        file_put_contents($this->file, $this->xml->asXML());
    }

    public function get(string $str): ?string
    {
        $matches = $this->xml->xpath("/conf/item[@name=\"$str\"]");
        if (count($matches)) {
            $this->lastmatch = $matches[0];
            return (string)$matches[0];
        }
        return null;
    }

    public function set(string $key, string $value): void
    {
        if (! is_null($this->get($key))) {
            $this->lastmatch[0] = $value;
            return;
        }
        $conf = $this->xml->conf;
        $this->xml->addChild('item', $value)->addAttribute('name', $key);
    }
}
